#!/usr/bin/perl
# 
# Script for transforming LingIt's transcribe format to two standard mlfs
# with attribute (manner and place) transcription. 
# The script presumes that the mapping from NAFTA phones to phonetic attributes
# is described in the files $Manner and $Place, which can be generated
# using the script MP_generate.pl
#
# TSv 

$infile="part_1.trans";
$Manner="Manner_List.txt";
$Place="Place_List.txt";

$outfile_man="part_1_manner.mlf";
$outfile_place="part_1_place.mlf";
$labfile_man="labels_man.list";
$labfile_place="labels_place.list";
$scale=10000;

$Manner="Manner_List.txt";
$Place="Place_List.txt";
%Manlist=();
%Plalist=();


# Read in the label transformations
open(MAN,"$Manner") || die "Could not open $Manner for reading\n";
open(PLA,"$Place") || die "Could not open $Place for reading\n";
while(<MAN>){
	chomp;
	($phone,$attr)=split(/ /,$_);
#	push(@{$Manlist{$attr}},$phone);
	$Manlist{$phone}=$attr;
}
close(MAN);
while(<PLA>){
	chomp;
	($phone,$attr)=split(/ /,$_);
	$Plalist{$phone}=$attr;
}
close(PLA);




open(IN,"$infile") || die "Could not open $infile for reading\n";
open(OUTMAN,">$outfile_man") || die "Could not open $outfile_man for writing\n";
open(OUTPLA,">$outfile_place") || die "Could not open $outfile_pla for writing\n";

%labels_man=();
%labels_place=();

while(<IN>){
	chomp;
    # Some stuff to convert to MLF format
	if ($_ =~ /TRANSCRIPTION/){
		print OUTMAN "#!MLF!#\n";
		print OUTPLA "#!MLF!#\n";
		next;
	}
	if ($_ =~ /part_1/){
		#print OUT "$_\n";
		$text= $_;
		$text =~ s/.*\///;
		substr($text, -1) = '';
		print OUTMAN "\"\*\/$text.lab\"\n";
		print OUTPLA "\"\*\/$text.lab\"\n";
		next;
	}
	if ($_ eq "."){
		print OUTMAN ".\n";
		print OUTPLA ".\n";
		next;
	}
	@line=split(/\t/,$_);
    
    # Scale timing info
	$line[0]=$scale*$line[0];
	$line[1]=$scale*$line[1];
    
    # Remove stress info
	$line[2] =~ s/\"//g;
	$line[2] =~ s/\%//g;
    
    # Remove non-linguistic info
	if ($line[2] =~ /^</){
		$line[2]="sil";
		$manattr="sil";
		$plaattr="sil";
	}
	if ($line[3] =~ /^</){delete($line[3]);}
    #
    # Transformation of symbols to attributes
    #
	if ($line[2] ne "sil"){
		$manattr=$Manlist{$line[2]};
		$plaattr=$Plalist{$line[2]};
	}
	if ($manattr eq ""){print "In-line: $_ and phone is $line[2]!!\n"}
	$labels_man{$manattr}=1;
	$labels_place{$plaattr}=1;

	print OUTMAN "$line[0]";
	print OUTMAN " $line[1]";
	print OUTMAN " $manattr";
	for ($i=3;$i<=2;$i++){print OUTMAN "$line[$i]";}
	print OUTMAN "\n";
	print OUTPLA "$line[0]";
	print OUTPLA " $line[1]";
	print OUTPLA " $plaattr";
	for ($i=3;$i<=2;$i++){print OUTPLA "$line[$i]";}
	print OUTPLA "\n";
}
close(IN);
close(OUTMAN);
close(OUTPLA);
open(LAB,">$labfile_man") || die "Could not open $labfile_man for writing\n";
foreach $lab (sort keys %labels_man){
	print LAB "$lab\n";
}
close(LAB);
open(LAB,">$labfile_place") || die "Could not open $labfile_place for writing\n";
foreach $lab (sort keys %labels_place){
	print LAB "$lab\n";
}
close(LAB);
