#!/usr/bin/perl

if($ARGV[0] eq "" || $ARGV[1] eq "" || $ARGV[2] eq "" || $ARGV[3] eq "")
{
	printf "SYNTAX: add_labels_to_pfile.pl pfile list_of_files mlf dict [tmp_dir] [frame_shift]\n";
	exit(1);
}

$pfile = $ARGV[0];
$list_of_files = $ARGV[1];
$mlf = $ARGV[2];
$dict = $ARGV[3];
$tmp_dir = $ARGV[4];
$frame_shift = $ARGV[5];
$to_frames = 1e-5;

if($frame_shift ne "")
{
    $to_frames = 1e-5 * 10 / $ARGV[5];
}
 
$tmp_dir = "/tmp" if($tmp_dir eq "");

# load dictionary = conversion table between phonemes and numbers
open DICT, "$dict" or die "ERROR: Can not open the dictionary: $dict.";
$i = 0;
while($l = <DICT>)
{
	chomp $l;
	$cnv_table{$l} = $i;
	$i++;
}
close DICT;

#load file names
open LIST, "$list_of_files" or die "ERROR: Can not open the list of files: $list_of_files.";
$i = 0;
while($l = <LIST>)
{
	chomp $l;
	if($l =~ /\/([^\/]+$)/)          #extract path
	{
		$fname = $1;
	}
	else
	{
		$fname = $l;
	}
	if($fname =~ /^(.+)\.[^\.]+$/)   #extract extension
	{
		$fname = $1;
	}
	$files_table{$i} = $fname;
	$i++;
}
close LIST;

# create list of sentences with their numbers of frames
$comm="pfile_info $pfile -p | tail -n +3 > $tmp_dir/frame_nums$$";
system($comm);

# load numbers of frames
open FNL, "$tmp_dir/frame_nums$$" or die "ERROR: Can not open the list of sentences with their numbers of frames: $tmp_dir/frame_nums$$.";
while($l = <FNL>)
{
	chomp $l;
	($sentence, $nframes) = split /\s+/, $l;
	$frames_table{$sentence} = $nframes;
}
close FNL;

system("rm $tmp_dir/frame_nums$$");

# load and parse MLF
open MLF, "$mlf" or die "ERROR: Can not open the MLF: $mlf.";
$header  = <MLF>;
while($file = <MLF>)
{
	chomp $file;
	$file =~ /\"(.+)\"/;
	$file = $1;
	if($file =~ /\/([^\/].+)$/)          #extract path
	{
		$file = $1;
	}
	if($file =~ /^(.+)\.[^\.]+$/)        #extract extension
	{
		$file = $1;
	}

	@starts_array = ();
	@stops_array = ();
	@phonemes_array = ();

	do
	{
		$l = <MLF>;
		chomp $l;
		($start, $stop, $phn, $oth) = split /\s+/, $l, 4;
#		printf " $phn";
		if($l ne ".")
		{
			push @starts_array, $start;
			push @stops_array, $stop;
			push @phonemes_array, $phn;
		}
	}
	while($l ne ".");
#	printf "\n\n---";
#	printf "@phonemes_array\n\n";
	$starts{$file} = "@starts_array";
	$stops{$file} = "@stops_array";
	$phonemes{$file} = "@phonemes_array";
}
close MLF;

# go file by file (sentence by sentence) and create labels
open LAB, ">$tmp_dir/labels$$";
$" = "\n";
for($i = 0; $i < (keys %files_table) + 0; $i++)
{
	$file = $files_table{$i};
	die "ERROR: There is less file names in the file list than the number of sections in the pfile." if(not defined($files_table{$i}));
	$nframes = $frames_table{$i};

	#printf "$file\n";

	die "ERROR: There are not labels in the MLF for the file: $file." if(not defined($phonemes{$file}));
	@starts_array = split / /, $starts{$file};
	@stops_array = split / /, $stops{$file};
	@phonemes_array = split / /, $phonemes{$file};

	$pstop = 0;
	$act_nframes = 0;
	$phn = "";
	for($j = 0; $j < @phonemes_array + 0; $j++)
	{
		$start = int($starts_array[$j] * $to_frames);
		$stop = int($stops_array[$j] * $to_frames);
		if($pstop != $start)
		{
			print "WARNING: Discontinuity - frames $pstop:$start in file $file.\n";
			$phn = "oth";
			die "ERROR: Phoneme '$phn' is not in your dictionaty." if(not defined($cnv_table{$phn}));
			$idx = $cnv_table{$phn};
			$n = $start - $pstop;
			if($n != 0)
			{
				$n = $nframes - $act_nframes if($act_nframes + $n > $nframes);  # Just to be sure that the label string is not longet than number of frame
				$act_nframes += $n;
				$str = "$idx " x $n;
				@new_labels = split / /, $str;
				print LAB "@new_labels\n";
			}
		}
		$phn = $phonemes_array[$j];
#		printf "$start $stop $phn $nframes\n";
		die "ERROR: Phoneme '$phn' is not in your dictionaty." if(not defined($cnv_table{$phn}));
		$idx = $cnv_table{$phn};
		$n = $stop - $start;
		$n = $nframes - $act_nframes if($act_nframes + $n > $nframes);
		if($n != 0)
		{
			$act_nframes += $n;
			$str = "$idx " x $n;
			@new_labels = split / /, $str;
			print LAB "@new_labels\n";
		}
		$pstop = $stop;
	}

	printf "WARNING: The feature file '$file' is shorter than labels: $nframes / $stop.\n" if($stop > $nframes);

	if($stop < $nframes)
	{
		$n = $nframes - $stop;
		$phn = "oth" if($n >= 3);	# 1 to 2 frames - let's suppose to be mistake in rounding and merge it with the last phoneme (usually 'pau')
		$phn = "oth" if($phn eq "");
		$n = $nframes - $act_nframes;
		$act_nframes += $n;
		$str = "$idx " x $n;
		@new_labels = split / /, $str;
		print LAB "@new_labels\n";
		printf "WARNING: The feature file '$file' is longer then the labels: $nframes / $stop.\n";
	}
}
close LAB;

#patch labels in the pfile
$comm = "pfile_patchlabels -i $pfile -o $tmp_dir/pfile$$ -l $tmp_dir/labels$$";
system($comm);

system("rm $tmp_dir/labels$$");
system("rm $pfile");
system("mv $tmp_dir/pfile$$ $pfile");


