#!/usr/bin/perl
#
#Script for diving posterior in a kaldi archive (.ark) by a given set of priors
#but final result exported in the .ark kaldi-format.
#In practice, logpriors are subtracted from logposteriors
#Inspired by TSv's post2pfile.pl
#------------Øystein Staven, NTNU 22 May 2016-----------------------------

###################################################################################
##	Declarations
###################################################################################
use Math::BigFloat;
Math::BigFloat->precision(-5);
###################################################################################
##	Check input arguments
###################################################################################
$num_args = $#ARGV+1;
if ($num_args != 3) {
    print "\nUsage: prior_ark.pl <input ark> <output ark> <priorlist>\n";
    exit;
}
###################################################################################
##	Input arguments and some conversions
###################################################################################
$inputark=$ARGV[0];
$tempinputark="temp_inputark.txt";
system("copy-feats ark:$inputark ark,t:$tempinputark");

$outputark=$ARGV[1];
$tempoutputark="temp_outputark.txt";
$filelist="temp_files";
$priorlist=$ARGV[2];


###################################################################################
##	Initialize
###################################################################################

open(INPUT,"$tempinputark") || die "Could not open $tempinputark\n";
open(OUTPUTTEMP,">$tempoutputark") || die "Could not open $tempoutputark for \n";
$count=0;
$frames=0;
@feats=();
@sourcefiles=();
#read priors into array for later use
@priors=();
	@priors = do {
	    open $fh, "<", $priorlist
	        or die "could not open $priorlist: $!";
	    <$fh>;
	};

close($fh);


# The main loop, reading the input file line by line
while($line = <INPUT>){
	chomp $line; #removel linebreaks
	$line =~ s/^ //g; #remove trailing whitespace
	
	#if line contains [ then it's the start of a new file
	if ($line =~ m/\[/) {

		print OUTPUTTEMP "$line\n ";
	}
	
	unless ($line =~ m/\[/ || $line =~ m/\]/){
		my @feats= split / /, $line;

	     for($el=0;$el<=$#feats-1;$el++) {
			$newfeat=$feats[$el]-$priors[$el];
	     	print OUTPUTTEMP "$newfeat ";
	     }
		 $newfeat=$feats[$el]-$priors[$el];
		 print OUTPUTTEMP "$newfeat \n ";
	}
	#last frame needs special treatment
	if ($line =~ m/\]/) {
		my @feats= split / /, $line;
		pop(@feats); #remove last element, aka ]
		
     for($el=0;$el<=$#feats-1;$el++) {
		$newfeat=$feats[$el]-$priors[$el-1];
     	print OUTPUTTEMP "$newfeat ";
     }
	  $newfeat=$feats[$el]-$priors[$el-1];
		print OUTPUTTEMP "$newfeat ]\n";
	}
	

}

close(INPUT);
close(OUTPUTTEMP);


	
system("copy-feats --binary=true ark,t:$tempoutputark ark:$outputark");
unlink("$tempinputark");
unlink("$tempoutputark");