#!/bin/perl
#This script calls on pfile_print to find prior probabilities associated with each class
#------------Øystein Staven, NTNU 22 May 2016-----------------------------

###################################################################################
##	Declarations
###################################################################################
use Math::BigFloat;
Math::BigFloat->precision(-8);
###################################################################################
##	Check input arguments
###################################################################################
$num_args = $#ARGV+1;
if ($num_args != 3) {
    print "\nUsage: prior_pfile.pl <input pfile> <dict> <output file>\n";
    exit;
}

###################################################################################
##	Input arguments
###################################################################################
$pfile=$ARGV[0];
$dict=$ARGV[1];
$output=$ARGV[2];
###################################################################################
##	Get info about number of states/attributes and make a txt version of pfile
###################################################################################
$num_attr=`cat $dict | wc -l`;

system("pfile_print -i $pfile -o temppfile.txt"); #TODO:uncomment

$pfileinfo=`pfile_info -i $pfile`;
$num_frames = (split / /, $pfileinfo)[2];
#print "$num_frames\n";
###################################################################################
## Initialize all count_array elements to zero. Needed later.####
###################################################################################
my @labellist=();
my @logprior=();
###################################################################################
##	Load entire pfile into array
###################################################################################
open(TEXTFILE,"temppfile.txt") or die "ERROR: can not open temppfile.txt\n";
while($line = <TEXTFILE>) {
	chomp $line;
	#now starts count process for each label
	$curlabel=(split / /, $line)[50];
	#increase at appropriate index
	$labellist[$curlabel]=$labellist[$curlabel] + 1;
}
close(TEXTFILE);
###################################################################################
##	Now find log priors and print to output
###################################################################################
open(OUTFILE, ">$output") or die "ERROR: can not open $output\n";
for($el=0;$el<=$num_attr-1;$el++) {

 if ($labellist[$el]  eq "")
  { $logprior[$el] = -1000; }
 else
 { $logprior[$el] = log($labellist[$el]) - log($num_frames); }
printf OUTFILE "%.5f\n", $logprior[$el];
}
close (OUTFILE);
###################################################################################
##Check if all probabilities sum to one.
###################################################################################
open $fh, "<", "$output" or die "Can't open '$output'\n";
my $probsum=Math::BigFloat->new(0.0);
while($line = <$fh>)
{
 chomp $line;
 $probsum=$probsum+exp($line);
}
 print "\n-------------------Sum of all probabilities is: $probsum------------------------\n";

 close $fh;
 unlink("temppfile.txt");